#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only
from oc_ocdm.support.support import get_datatype_from_iso_8601

if TYPE_CHECKING:
    from typing import Optional, List
    from rdflib import URIRef
    from oc_ocdm.graph.entities.bibliographic.bibliographic_reference import BibliographicReference
    from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole
    from oc_ocdm.graph.entities.bibliographic.discourse_element import DiscourseElement
    from oc_ocdm.graph.entities.bibliographic.resource_embodiment import ResourceEmbodiment
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class BibliographicResource(BibliographicEntity):
    """Bibliographic resource (short: br): a published bibliographic resource that cites/is
       cited by another published bibliographic resource."""

    @accepts_only('br')
    def merge(self, other: BibliographicResource) -> None:
        super(BibliographicResource, self).merge(other)

        title: Optional[str] = other.get_title()
        if title is not None:
            self.has_title(title)

        subtitle: Optional[str] = other.get_subtitle()
        if subtitle is not None:
            self.has_subtitle(subtitle)

        container: Optional[BibliographicResource] = other.get_is_part_of()
        if container is not None:
            self.is_part_of(container)

        citations_list: List[BibliographicResource] = other.get_citations()
        for cur_citation in citations_list:
            self.has_citation(cur_citation)

        pub_date: Optional[str] = other.get_pub_date()
        if pub_date is not None:
            self.has_pub_date(pub_date)

        re_list: List[ResourceEmbodiment] = other.get_formats()
        for cur_format in re_list:
            self.has_format(cur_format)

        number: Optional[str] = other.get_number()
        if number is not None:
            self.has_number(number)

        edition: Optional[str] = other.get_edition()
        if edition is not None:
            self.has_edition(edition)

        be_list: List[BibliographicReference] = other.get_contained_in_reference_lists()
        for reference in be_list:
            self.contains_in_reference_list(reference)

        de_list: List[DiscourseElement] = other.get_contained_discourse_elements()
        for discourse_element in de_list:
            self.contains_discourse_element(discourse_element)

        ar_list: List[AgentRole] = other.get_contributors()
        for agent_role in ar_list:
            self.has_contributor(agent_role)

        related_doc_list: List[URIRef] = other.get_related_documents()
        for doc in related_doc_list:
            self.has_related_document(doc)

    # HAS TITLE
    def get_title(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_title)

    @accepts_only('literal')
    def has_title(self, string: str) -> None:
        """The title of the bibliographic resource.
        """
        self.remove_title()
        self._create_literal(GraphEntity.iri_title, string)

    def remove_title(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_title, None))

    # HAS SUBTITLE
    def get_subtitle(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_subtitle)

    @accepts_only('literal')
    def has_subtitle(self, string: str) -> None:
        """The subtitle of the bibliographic resource.
        """
        self.remove_subtitle()
        self._create_literal(GraphEntity.iri_has_subtitle, string)

    def remove_subtitle(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_subtitle, None))

    # IS PART OF (BibliographicResource)
    def get_is_part_of(self) -> Optional[BibliographicResource]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_part_of, 'br')
        if uri is not None:
            return self.g_set.add_br(self.resp_agent, self.source, uri)

    @accepts_only('br')
    def is_part_of(self, br_res: BibliographicResource) -> None:
        """The corpus identifier of the bibliographic resource (e.g. issue, volume, journal,
        conference proceedings) that contains the subject bibliographic resource.
        """
        self.remove_is_part_of()
        self.g.add((self.res, GraphEntity.iri_part_of, br_res.res))

    def remove_is_part_of(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_part_of, None))

    # CITES (BibliographicResource)
    def get_citations(self) -> List[BibliographicResource]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_cites, 'br')
        result: List[BibliographicResource] = []
        for uri in uri_list:
            result.append(self.g_set.add_br(self.resp_agent, self.source, uri))
        return result

    @accepts_only('br')
    def has_citation(self, br_res: BibliographicResource) -> None:
        """The corpus identifier of the bibliographic resource cited by the subject bibliographic
        resource.
        """
        self.g.add((self.res, GraphEntity.iri_cites, br_res.res))

    @accepts_only('br')
    def remove_citation(self, br_res: BibliographicResource = None) -> None:
        if br_res is not None:
            self.g.remove((self.res, GraphEntity.iri_cites, br_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_cites, None))

    # HAS PUBLICATION DATE
    def get_pub_date(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_publication_date)

    @accepts_only('literal')
    def has_pub_date(self, string: str) -> None:
        """The date of publication of the bibliographic resource.
        """
        cur_type, string = get_datatype_from_iso_8601(string)
        if cur_type is not None and string is not None:
            self.remove_pub_date()
            self._create_literal(GraphEntity.iri_has_publication_date, string, cur_type, False)

    def remove_pub_date(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_publication_date, None))

    # IS EMBODIED AS (ResourceEmbodiment)
    def get_formats(self) -> List[ResourceEmbodiment]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_embodiment, 're')
        result: List[ResourceEmbodiment] = []
        for uri in uri_list:
            result.append(self.g_set.add_re(self.resp_agent, self.source, uri))
        return result

    @accepts_only('re')
    def has_format(self, re_res: ResourceEmbodiment) -> None:
        """The corpus identifier of the resource embodiment defining the format in which the
        bibliographic resource has been embodied, which can be either print or digital.
        """
        self.g.add((self.res, GraphEntity.iri_embodiment, re_res.res))

    @accepts_only('re')
    def remove_format(self, re_res: ResourceEmbodiment = None):
        if re_res is not None:
            self.g.remove((self.res, GraphEntity.iri_embodiment, re_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_embodiment, None))

    # HAS NUMBER
    def get_number(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_sequence_identifier)

    @accepts_only('literal')
    def has_number(self, string: str) -> None:
        """A literal (for example a number or a letter) that identifies the sequence position of the
        bibliographic resource as a particular item within a larger collection (e.g. an article
        number within a journal issue, a volume number of a journal, a chapter number within
        a book).
        """
        self.remove_number()
        self._create_literal(GraphEntity.iri_has_sequence_identifier, string)

    def remove_number(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_sequence_identifier, None))

    # HAS EDITION
    def get_edition(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_edition)

    @accepts_only('literal')
    def has_edition(self, string: str) -> None:
        """An identifier for one of several alternative editions of a particular bibliographic
        resource.
        """
        self.remove_edition()
        self._create_literal(GraphEntity.iri_has_edition, string)

    def remove_edition(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_edition, None))

    # HAS PART (BibliographicReference)
    def get_contained_in_reference_lists(self) -> List[BibliographicReference]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_contains_reference, 'be')
        result: List[BibliographicReference] = []
        for uri in uri_list:
            result.append(self.g_set.add_be(self.resp_agent, self.source, uri))
        return result

    @accepts_only('be')
    def contains_in_reference_list(self, be_res: BibliographicReference) -> None:
        """A bibliographic reference within the bibliographic resource, or a discourse element
        wherein the text of the bibliographic resources can be organized.
        """
        self.g.add((self.res, GraphEntity.iri_contains_reference, be_res.res))

    @accepts_only('be')
    def remove_contained_in_reference_list(self, be_res: BibliographicReference = None) -> None:
        if be_res is not None:
            self.g.remove((self.res, GraphEntity.iri_contains_reference, be_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_contains_reference, None))

    # HAS PART (DiscourseElement)
    def get_contained_discourse_elements(self) -> List[DiscourseElement]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_contains_de, 'de')
        result: List[DiscourseElement] = []
        for uri in uri_list:
            result.append(self.g_set.add_de(self.resp_agent, self.source, uri))
        return result

    @accepts_only('de')
    def contains_discourse_element(self, de_res: DiscourseElement) -> None:
        """A bibliographic reference within the bibliographic resource, or a discourse element
        wherein the text of the bibliographic resources can be organized.
        """
        self.g.add((self.res, GraphEntity.iri_contains_de, de_res.res))

    @accepts_only('de')
    def remove_contained_discourse_element(self, de_res: DiscourseElement = None) -> None:
        if de_res is not None:
            self.g.remove((self.res, GraphEntity.iri_contains_de, de_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_contains_de, None))

    # HAS CONTRIBUTOR (AgentRole)
    def get_contributors(self) -> List[AgentRole]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_is_document_context_for, 'ar')
        result: List[AgentRole] = []
        for uri in uri_list:
            result.append(self.g_set.add_ar(self.resp_agent, self.source, uri))
        return result

    @accepts_only('ar')
    def has_contributor(self, ar_res: AgentRole):
        self.g.add((self.res, GraphEntity.iri_is_document_context_for, ar_res.res))

    @accepts_only('ar')
    def remove_contributor(self, ar_res: AgentRole = None):
        if ar_res is not None:
            self.g.remove((self.res, GraphEntity.iri_is_document_context_for, ar_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_is_document_context_for, None))

    # HAS RELATED DOCUMENT
    def get_related_documents(self) -> List[URIRef]:
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_relation)
        return uri_list

    @accepts_only('thing')
    def has_related_document(self, thing_ref: URIRef) -> None:
        """A document external to the Corpus, that is related to the bibliographic resource (such
        as a version of the bibliographic resource – for example a preprint – recorded in an
        external database).
        """
        self.g.add((self.res, GraphEntity.iri_relation, thing_ref))

    @accepts_only('thing')
    def remove_related_document(self, thing_ref: URIRef = None) -> None:
        if thing_ref is not None:
            self.g.remove((self.res, GraphEntity.iri_relation, thing_ref))
        else:
            self.g.remove((self.res, GraphEntity.iri_relation, None))

    # HAS TYPE
    def create_archival_document(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_archival_document)

    def create_book(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book)

    def create_book_chapter(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book_chapter)

    def create_book_part(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_part)

    def create_book_section(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_expression_collection)

    def create_book_series(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book_series)

    def create_book_set(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book_set)

    def create_book_track(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_expression)

    def create_component(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_expression)

    def create_dataset(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_data_file)

    def create_dissertation(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_thesis)

    def create_edited_book(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book)

    def create_journal_article(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_journal_article)

    def create_issue(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_journal_issue)

    def create_volume(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_journal_volume)

    def create_journal(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_journal)

    def create_monograph(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_book)

    def create_proceedings_article(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_proceedings_paper)

    def create_proceedings(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_academic_proceedings)

    def create_reference_book(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_reference_book)

    def create_reference_entry(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_reference_entry)

    def create_report_series(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_series)

    def create_report(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_report_document)

    def create_standard_series(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_series)

    def create_standard(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_specification_document)

    def create_series(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_series)

    def create_expression_collection(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_expression_collection)

    def create_other(self) -> None:
        """The type of the bibliographic resource
        """
        self._create_type(GraphEntity.iri_expression)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

import re
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.decorators import accepts_only
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class ResourceEmbodiment(BibliographicEntity):
    """Resource embodiment (short: re): the particular physical or digital format in which a
       bibliographic resource was made available by its publisher."""

    @accepts_only('re')
    def merge(self, other: ResourceEmbodiment) -> None:
        super(ResourceEmbodiment, self).merge(other)

        media_type: Optional[URIRef] = other.get_media_type()
        if media_type is not None:
            self.has_media_type(media_type)

        starting_page: Optional[str] = other.get_starting_page()
        if starting_page is not None:
            self.has_starting_page(starting_page)

        ending_page: Optional[str] = other.get_ending_page()
        if ending_page is not None:
            self.has_ending_page(ending_page)

        url: Optional[URIRef] = other.get_url()
        if url is not None:
            self.has_url(url)

    # HAS FORMAT
    def get_media_type(self) -> Optional[URIRef]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_format)
        return uri

    @accepts_only('thing')
    def has_media_type(self, thing_ref: URIRef) -> None:
        """It allows one to specify the IANA media type of the embodiment.
        """
        self.remove_media_type()
        self.g.add((self.res, GraphEntity.iri_has_format, thing_ref))

    def remove_media_type(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_format, None))

    # HAS FIRST PAGE
    def get_starting_page(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_starting_page)

    @accepts_only('literal')
    def has_starting_page(self, string: str) -> None:
        """The first page of the bibliographic resource according to the current embodiment.
        """
        self.remove_starting_page()
        if re.search("[-–]+", string) is None:
            page_number = string
        else:
            page_number = re.sub("[-–]+.*$", "", string)
        self._create_literal(GraphEntity.iri_starting_page, page_number)

    def remove_starting_page(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_starting_page, None))

    # HAS LAST PAGE
    def get_ending_page(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_ending_page)

    @accepts_only('literal')
    def has_ending_page(self, string: str) -> None:
        """The last page of the bibliographic resource according to the current embodiment.
        """
        self.remove_ending_page()
        if re.search("[-–]+", string) is None:
            page_number = string
        else:
            page_number = re.sub("^.*[-–]+", "", string)
        self._create_literal(GraphEntity.iri_ending_page, page_number)

    def remove_ending_page(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_ending_page, None))

    # HAS URL
    def get_url(self) -> Optional[URIRef]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_url)
        return uri

    @accepts_only('thing')
    def has_url(self, thing_ref: URIRef) -> None:
        """The URL at which the embodiment of the bibliographic resource is available.
        """
        self.remove_url()
        self.g.add((self.res, GraphEntity.iri_has_url, thing_ref))

    def remove_url(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_url, None))

    # HAS TYPE
    def create_digital_embodiment(self) -> None:
        """It identifies the particular type of the embodiment, either digital or print.
        """
        self._create_type(GraphEntity.iri_digital_manifestation)

    def create_print_embodiment(self) -> None:
        """It identifies the particular type of the embodiment, either digital or print.
        """
        self._create_type(GraphEntity.iri_print_object)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

from oc_ocdm.abstract_entity import AbstractEntity

if TYPE_CHECKING:
    from typing import List, ClassVar, Dict, Optional
    from rdflib import URIRef, Graph


class AbstractSet(ABC):

    short_name_to_type_iri: ClassVar[Dict[str, URIRef]] = {}

    def __init__(self) -> None:
        self.res_to_entity: Dict[URIRef, AbstractEntity] = {}

    def graphs(self) -> List[Graph]:
        result: List[Graph] = []
        for entity in self.res_to_entity.values():
            if len(entity.g) > 0:
                result.append(entity.g)
        return result

    @abstractmethod
    def get_entity(self, res: URIRef) -> Optional[AbstractEntity]:
        pass

    @staticmethod
    def get_graph_iri(g: Graph) -> str:
        return str(g.identifier)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from rdflib import XSD

from oc_ocdm.decorators import accepts_only
from oc_ocdm.metadata.metadata_entity import MetadataEntity

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef


class Distribution(MetadataEntity):
    """Distribution (short: di): an accessible form of a dataset, for example a downloadable
       file."""

    @accepts_only('di')
    def merge(self, other: Distribution) -> None:
        super(Distribution, self).merge(other)

        title: Optional[str] = other.get_title()
        if title is not None:
            self.has_title(title)

        description: Optional[str] = other.get_description()
        if description is not None:
            self.has_description(description)

        pub_date: Optional[str] = other.get_publication_date()
        if pub_date is not None:
            self.has_publication_date(pub_date)

        byte_size: Optional[str] = other.get_byte_size()
        if byte_size is not None:
            self.has_byte_size(byte_size)

        license_uri: Optional[URIRef] = other.get_license()
        if license_uri is not None:
            self.has_license(license_uri)

        download_url: Optional[URIRef] = other.get_download_url()
        if download_url is not None:
            self.has_download_url(download_url)

        media_type: Optional[URIRef] = other.get_media_type()
        if media_type is not None:
            self.has_media_type(media_type)

    # HAS TITLE
    def get_title(self) -> Optional[str]:
        return self._get_literal(MetadataEntity.iri_title)

    @accepts_only('literal')
    def has_title(self, string: str) -> None:
        """The title of the distribution."""
        self.remove_title()
        self._create_literal(MetadataEntity.iri_title, string)

    def remove_title(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_title, None))

    # HAS DESCRIPTION
    def get_description(self) -> Optional[str]:
        return self._get_literal(MetadataEntity.iri_description)

    @accepts_only('literal')
    def has_description(self, string: str) -> None:
        """A short textual description of the content of the distribution."""
        self.remove_description()
        self._create_literal(MetadataEntity.iri_description, string)

    def remove_description(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_description, None))

    # HAS PUBLICATION DATE
    def get_publication_date(self) -> Optional[str]:
        return self._get_literal(MetadataEntity.iri_issued)

    @accepts_only('literal')
    def has_publication_date(self, string: str) -> None:
        """The date of first publication of the distribution."""
        self.remove_publication_date()
        self._create_literal(MetadataEntity.iri_issued, string, XSD.dateTime, False)

    def remove_publication_date(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_issued, None))

    # HAS BYTE SIZE
    def get_byte_size(self) -> Optional[str]:
        return self._get_literal(MetadataEntity.iri_byte_size)

    @accepts_only('literal')
    def has_byte_size(self, string: str) -> None:
        """The size in bytes of the distribution."""
        self.remove_byte_size()
        self._create_literal(MetadataEntity.iri_byte_size, string, XSD.decimal)

    def remove_byte_size(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_byte_size, None))

    # HAS LICENSE
    def get_license(self) -> Optional[URIRef]:
        return self._get_literal(MetadataEntity.iri_license)

    @accepts_only('thing')
    def has_license(self, thing_res: URIRef) -> None:
        """The resource describing the license associated with the data in the distribution."""
        self.remove_license()
        self.g.add((self.res, MetadataEntity.iri_license, thing_res))

    def remove_license(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_license, None))

    # HAS DOWNLOAD URL
    def get_download_url(self) -> Optional[URIRef]:
        return self._get_literal(MetadataEntity.iri_download_url)

    @accepts_only('thing')
    def has_download_url(self, thing_res: URIRef) -> None:
        """The URL of the document where the distribution is stored."""
        self.remove_download_url()
        self.g.add((self.res, MetadataEntity.iri_download_url, thing_res))

    def remove_download_url(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_download_url, None))

    # HAS_MEDIA_TYPE
    def get_media_type(self) -> Optional[URIRef]:
        return self._get_literal(MetadataEntity.iri_media_type)

    @accepts_only('thing')
    def has_media_type(self, thing_res: URIRef) -> None:
        """The file type of the representation of the distribution (according to IANA media types)."""
        self.remove_media_type()
        self.g.add((self.res, MetadataEntity.iri_media_type, thing_res))

    def remove_media_type(self) -> None:
        self.g.remove((self.res, MetadataEntity.iri_media_type, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef
    from oc_ocdm.graph.entities.bibliographic.citation import Citation
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class ReferenceAnnotation(BibliographicEntity):
    """Reference annotation (short: an): an annotation, attached either to an in-text
       reference pointer or to a bibliographic reference, describing the related citation. If an
       in-text reference pointer is annotated, the related citation may be characterized with a
       citation function (the reason for that citation) specific to the textual location of that
       in-text reference pointer within the citing entity. If a bibliographic reference is
       annotated, the related citation may be similarly characterized in a more general way
       with a citation function (the reason for that citation)."""

    @accepts_only('an')
    def merge(self, other: ReferenceAnnotation) -> None:
        super(ReferenceAnnotation, self).merge(other)

        citation: Optional[Citation] = other.get_body_annotation()
        if citation is not None:
            self.has_body_annotation(citation)

    # HAS CITATION (Citation)
    def get_body_annotation(self) -> Optional[Citation]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_body, 'ci')
        if uri is not None:
            return self.g_set.add_ci(self.resp_agent, self.source, uri)

    @accepts_only('ci')
    def has_body_annotation(self, ci_res: Citation) -> None:
        """The citation to which the annotation relates, that is relevant either to a bibliographic
        reference or to an in-text reference pointer that denotes such a bibliographic reference.
        """
        self.remove_body_annotation()
        self.g.add((self.res, GraphEntity.iri_has_body, ci_res.res))

    def remove_body_annotation(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_body, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from rdflib import XSD

from oc_ocdm.decorators import accepts_only
from oc_ocdm.support.support import get_datatype_from_iso_8601

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef
    from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import BibliographicResource
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class Citation(BibliographicEntity):
    """Citation (short: ci): a permanent conceptual directional link from the citing
       bibliographic resource to a cited bibliographic resource. A citation is created by the
       performative act of an author citing a published work that is relevant to the current
       work by using a particular textual device. Typically, citations are made by including a
       bibliographic reference in the reference list of the citing work and by denoting such a
       bibliographic reference using one or more in-text reference pointers (e.g. '[1]'), or by
       the inclusion within the citing work of a link, in the form of an HTTP Uniform Resource
       Locator (URL), to the cited bibliographic resource on the World Wide Web."""

    @accepts_only('ci')
    def merge(self, other: Citation) -> None:
        super(Citation, self).merge(other)

        citing_br: Optional[BibliographicResource] = other.get_citing_entity()
        if citing_br is not None:
            self.has_citing_entity(citing_br)

        cited_br: Optional[BibliographicResource] = other.get_cited_entity()
        if cited_br is not None:
            self.has_cited_entity(cited_br)

        creation_date: Optional[str] = other.get_citation_creation_date()
        if creation_date is not None:
            self.has_citation_creation_date(creation_date)

        time_span: Optional[str] = other.get_citation_time_span()
        if time_span is not None:
            self.has_citation_time_span(time_span)

        characterization: Optional[URIRef] = other.get_citation_characterization()
        if characterization is not None:
            self.has_citation_characterization(characterization)

    # HAS CITING DOCUMENT (BibliographicResource)
    def get_citing_entity(self) -> Optional[BibliographicResource]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_citing_entity, 'br')
        if uri is not None:
            return self.g_set.add_br(self.resp_agent, self.source, uri)

    @accepts_only('br')
    def has_citing_entity(self, citing_res: BibliographicResource) -> None:
        """The bibliographic resource which acts as the source for the citation.
        """
        self.remove_citing_entity()
        self.g.add((self.res, GraphEntity.iri_has_citing_entity, citing_res.res))

    def remove_citing_entity(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_citing_entity, None))

    # HAS CITED DOCUMENT (BibliographicResource)
    def get_cited_entity(self) -> Optional[BibliographicResource]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_cited_entity, 'br')
        if uri is not None:
            return self.g_set.add_br(self.resp_agent, self.source, uri)

    @accepts_only('br')
    def has_cited_entity(self, cited_res: BibliographicResource) -> None:
        """ The bibliographic resource which acts as the target for the citation.
        """
        self.remove_cited_entity()
        self.g.add((self.res, GraphEntity.iri_has_cited_entity, cited_res.res))

    def remove_cited_entity(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_cited_entity, None))

    # HAS CITATION CREATION DATE
    def get_citation_creation_date(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_citation_creation_date)

    @accepts_only('literal')
    def has_citation_creation_date(self, string: str) -> None:
        """The date on which the citation was created. This has the same numerical value
        as the publication date of the citing bibliographic resource, but is a property
        of the citation itself. When combined with the citation time span, it permits
        that citation to be located in history.
        """
        cur_type, string = get_datatype_from_iso_8601(string)
        if cur_type is not None and string is not None:
            self.remove_citation_creation_date()
            self._create_literal(GraphEntity.iri_has_citation_creation_date, string, cur_type, False)

    def remove_citation_creation_date(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_citation_creation_date, None))

    # HAS CITATION TIME SPAN
    def get_citation_time_span(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_citation_time_span)

    @accepts_only('literal')
    def has_citation_time_span(self, string: str) -> None:
        """The date interval between the publication date of the cited bibliographic resource and
        the publication date of the citing bibliographic resource.
        """
        self.remove_citation_time_span()
        self._create_literal(GraphEntity.iri_has_citation_time_span, string, XSD.duration, False)

    def remove_citation_time_span(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_citation_time_span, None))

    # HAS CITATION CHARACTERIZATION
    def get_citation_characterization(self) -> Optional[URIRef]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_citation_characterisation)
        return uri

    @accepts_only('thing')
    def has_citation_characterization(self, thing_ref: URIRef) -> None:
        """The citation function characterizing the purpose of the citation.
        """
        self.remove_citation_characterization()
        self.g.add((self.res, GraphEntity.iri_citation_characterisation, thing_ref))

    def remove_citation_characterization(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_citation_characterisation, None))

    # HAS TYPE
    def create_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_self_citation)

    def create_affiliation_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_affiliation_self_citation)

    def create_author_network_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_author_network_self_citation)

    def create_author_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_author_self_citation)

    def create_funder_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_funder_self_citation)

    def create_journal_self_citation(self) -> None:
        self._create_type(GraphEntity.iri_journal_self_citation)

    def create_journal_cartel_citation(self) -> None:
        self._create_type(GraphEntity.iri_journal_cartel_citation)

    def create_distant_citation(self) -> None:
        self._create_type(GraphEntity.iri_distant_citation)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

import re
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.decorators import accepts_only
from oc_ocdm.support.support import is_string_empty, encode_url

"""
Notes about ID:

    HAS LITERAL VALUE and HAS SCHEME are generated by factory methods!
    Chill down, everything seems OK here!
"""


class Identifier(GraphEntity):
    """Identifier (short: id): an external identifier (e.g. DOI, ORCID, PubMedID, Open
       Citation Identifier) associated with the bibliographic entity. Members of this class of
       metadata are themselves given unique corpus identifiers e.g. 'id/0420129'."""

    @accepts_only('id')
    def merge(self, other: Identifier):
        super(Identifier, self).merge(other)

        literal_value: Optional[str] = other.get_literal_value()
        scheme: Optional[URIRef] = other.get_scheme()
        if literal_value is not None and scheme is not None:
            self._associate_identifier_with_scheme(literal_value, scheme)

    # HAS LITERAL VALUE and HAS SCHEME
    def get_literal_value(self) -> Optional[str]:
        return self._get_literal(GraphEntity.iri_has_literal_value)

    def get_scheme(self) -> Optional[URIRef]:
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_uses_identifier_scheme)
        return uri

    @accepts_only('literal')
    def create_oci(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_oci)

    @accepts_only('literal')
    def create_orcid(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_orcid)

    @accepts_only('literal')
    def create_doi(self, string: str) -> None:
        self._associate_identifier_with_scheme(string.lower(), GraphEntity.iri_doi)

    @accepts_only('literal')
    def create_pmid(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_pmid)

    @accepts_only('literal')
    def create_pmcid(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_pmcid)

    @accepts_only('literal')
    def create_issn(self, string: str) -> None:
        cur_string = re.sub("–", "-", string)
        if cur_string != "0000-0000":
            self._associate_identifier_with_scheme(string, GraphEntity.iri_issn)

    @accepts_only('literal')
    def create_isbn(self, string: str) -> None:
        self._associate_identifier_with_scheme(re.sub("–", "-", string), GraphEntity.iri_isbn)

    @accepts_only('literal')
    def create_url(self, string: str) -> None:
        self._associate_identifier_with_scheme(encode_url(string.lower()), GraphEntity.iri_url)

    @accepts_only('literal')
    def create_xpath(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_xpath)

    @accepts_only('literal')
    def create_intrepid(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_intrepid)

    @accepts_only('literal')
    def create_xmlid(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_xmlid)

    @accepts_only('literal')
    def create_wikidata(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_wikidata)

    @accepts_only('literal')
    def create_wikipedia(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_wikipedia)

    @accepts_only('literal')
    def create_crossref(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_crossref)

    @accepts_only('literal')
    def create_viaf(self, string: str) -> None:
        self._associate_identifier_with_scheme(string, GraphEntity.iri_viaf)

    def _associate_identifier_with_scheme(self, string: str, id_type: URIRef) -> None:
        if not is_string_empty(string):
            self.remove_identifier_with_scheme()
            self._create_literal(GraphEntity.iri_has_literal_value, string)
            self.g.add((self.res, GraphEntity.iri_uses_identifier_scheme, id_type))

    def remove_identifier_with_scheme(self) -> None:
        self.g.remove((self.res, GraphEntity.iri_has_literal_value, None))
        self.g.remove((self.res, GraphEntity.iri_uses_identifier_scheme, None))
